==> InfiniTec.Exchange_0.96_Source.zip: WebDav/Attachment.cs <==
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using InfiniTec.Exchange.WebDav;
using InfiniTec.Threading;
using System.ComponentModel;
using InfiniTec.Exchange.WebDav.Request;

namespace InfiniTec.Exchange.WebDav
{
    /// <summary>
    /// This class contains information about an attachment of an Exchange item.
    /// </summary>
    public class Attachment : ObjectBase
    {
        /// <summary>
        /// This event is executed when the status of the <see cref="Download"/> operation changes.
        /// </summary>
        public event ProgressChangedEventHandler<RequestStatus> DownloadProgressChanged;
        /// <summary>
        /// This event is executed when the <see cref="Download"/> operation is completed.
        /// </summary>
        public event AsyncCompletedEventHandler DownloadCompleted;

        private string _Name;
        private int _Size;
        private string _MimeType;
        private string _ContentDisposition;

        /// <summary>
        /// Returns the content disposition of the attachment ("attachment", "inline", etc). 
        /// </summary>
        public string ContentDisposition
        {
            get { return _ContentDisposition; }
        }
	
        /// <summary>
        /// Returns the MIME type of the attachment (for example "text/plain").
        /// </summary>
        public string MimeType
        {
            get { return _MimeType; }
        }
	
        private PropertyCollection _Properties;

        /// <summary>
        /// Provides access to the properties of the attachment.
        /// </summary>
        public PropertyCollection Properties
        {
            get { return _Properties; }
        }

        /// <summary>
        /// Returns the size of the attachment.
        /// </summary>
        public int Size
        {
            get { return _Size; }
        }

        /// <summary>
        /// Returns the name of the attachment.
        /// </summary>
        public string Name
        {
            get { return _Name; }
        }

        internal Attachment(Uri url, string name, int size, Connection connection)
            : base(url, connection)
        {
            _Name = name;
            _Size = size;
        }

        internal Attachment(PropertyContainer container, Connection connection): base(container.Url, connection)
        {
            Property<int> size;
            Property<string> stringProperty;

            _Properties = new PropertyCollection();
            _Properties.AddRange(container.Properties);

            if (Properties.TryGetProperty<string>(NamedProperties.Attachment.LongFilename, out stringProperty))
            {
                _Name = stringProperty.Value;
            }
            else if (Properties.TryGetProperty<string>(NamedProperties.Attachment.Filename, out stringProperty))
            {
                _Name = stringProperty.Value;
            }
            else if (Properties.TryGetProperty<string>(NamedProperties.Attachment.Filename2, out stringProperty))
            {
                _Name = stringProperty.Value;
            }
            else {
                _Name = "<unknown name>";
            }

            _Size = Properties.TryGetProperty<int>(NamedProperties.Attachment.Size, out size) ? size.Value : 0;
            _ContentDisposition = Properties.TryGetProperty<string>(NamedProperties.Attachment.ContentDisposition, out stringProperty) ? stringProperty.Value : "";
            _MimeType = Properties.TryGetProperty<string>(NamedProperties.Attachment.MimeType, out stringProperty) ? stringProperty.Value : "";
        }

        /// <summary>
        /// Returns an <see cref="InfiniTec.Threading.AsyncOperation"/> which can be used to download the attachment.
        /// </summary>
        /// <param name="outputStream">The target stream for the content of the item.</param>
        /// <returns>An <see cref="InfiniTec.Threading.AsyncOperation"/>.</returns>
        public AsyncOperation<RequestStatus> GetDownloadOperation(Stream outputStream)
        {
            return Methods.Get(Url, outputStream);
        }

        /// <summary>
        /// Downloads the content of the attachment into the specified output stream. The current thread is blocked
        /// until the operation completes.
        /// </summary>
        /// <param name="outputStream">A <see cref="Stream"/> used to store the contents of the attachment.</param>
        /// <exception cref="AsyncOperationFailedException">Thrown, when the operation fails. See <see cref="Exception.InnerException"/> for details.</exception>
        public void Download(Stream outputStream)
        {
            using (AsyncOperation<RequestStatus> operation = GetDownloadOperation(outputStream))
            {
                operation.Run();

                operation.WaitForCompletion();
            }
        }

        /// <summary>
        /// Asynchronously downloads the content of the attachment into the specified output stream.
        /// </summary>
        /// <param name="outputStream">A <see cref="Stream"/> used to store the contents of the attachment.</param>
        public void DownloadAsync(Stream outputStream)
        {
            AsyncOperation<RequestStatus> operation;

            InitiatePendingAsyncOperation();

            operation = GetDownloadOperation(outputStream);

            operation.ProgressChanged += DownloadProgressChanged;
            operation.Completed += Download_Completed;
            operation.Error += Download_Error;

            operation.Run();
        }

        void Download_Error(object sender, ExceptionEventArgs e)
        {
            UnregisterPendingAsyncOperation();

            OnDownloadCompleted(new AsyncCompletedEventArgs(e.Exception, false, null));
        }

        void Download_Completed(object sender, CompletedEventArgs e)
        {
            UnregisterPendingAsyncOperation();

            OnDownloadCompleted(new AsyncCompletedEventArgs(null, e.Status == OperationStatus.Aborted, null));
        }

        /// <summary>
        /// Executes the <see cref="DownloadCompleted"/> event.
        /// </summary>
        /// <param name="e">The event parameters.</param>
        protected virtual void OnDownloadCompleted(AsyncCompletedEventArgs e)
        {
            if (DownloadCompleted != null) DownloadCompleted(this, e);
        }
    }
}

==> InfiniTec.Exchange_0.97_Source.zip: WebDav/Attachment.cs <==
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using InfiniTec.Exchange.WebDav;
using InfiniTec.Threading;
using System.ComponentModel;
using InfiniTec.Exchange.WebDav.Request;

namespace InfiniTec.Exchange.WebDav
{
    /// <summary>
    /// This class contains information about an attachment of an Exchange item.
    /// </summary>
    public class Attachment : ObjectBase
    {
        /// <summary>
        /// This event is executed when the status of the <see cref="Download"/> operation changes.
        /// </summary>
        public event ProgressChangedEventHandler<RequestStatus> DownloadProgressChanged;
        /// <summary>
        /// This event is executed when the <see cref="Download"/> operation is completed.
        /// </summary>
        public event AsyncCompletedEventHandler DownloadCompleted;

        private string _Name;
        private int _Size;
        private string _MimeType;
        private string _ContentDisposition;

        /// <summary>
        /// Returns the content disposition of the attachment ("attachment", "inline", etc). 
        /// </summary>
        public string ContentDisposition
        {
            get { return _ContentDisposition; }
        }
	
        /// <summary>
        /// Returns the MIME type of the attachment (for example "text/plain").
        /// </summary>
        public string MimeType
        {
            get { return _MimeType; }
        }
	
        private PropertyCollection _Properties;

        /// <summary>
        /// Provides access to the properties of the attachment.
        /// </summary>
        public PropertyCollection Properties
        {
            get { return _Properties; }
        }

        /// <summary>
        /// Returns the size of the attachment.
        /// </summary>
        public int Size
        {
            get { return _Size; }
        }

        /// <summary>
        /// Returns the name of the attachment.
        /// </summary>
        public string Name
        {
            get { return _Name; }
        }

        internal Attachment(Uri url, string name, int size, Connection connection)
            : base(url, connection)
        {
            _Name = name;
            _Size = size;
        }

        internal Attachment(PropertyContainer container, Connection connection): base(container.Url, connection)
        {
            Property<int> size;
            Property<string> stringProperty;

            _Properties = new PropertyCollection();
            _Properties.AddRange(container.Properties);

            if (Properties.TryGetProperty<string>(NamedProperties.Attachment.LongFilename, out stringProperty))
            {
                _Name = stringProperty.Value;
            }
            else if (Properties.TryGetProperty<string>(NamedProperties.Attachment.Filename, out stringProperty))
            {
                _Name = stringProperty.Value;
            }
            else if (Properties.TryGetProperty<string>(NamedProperties.Attachment.Filename2, out stringProperty))
            {
                _Name = stringProperty.Value;
            }
            else {
                _Name = "<unknown name>";
            }

            _Size = Properties.TryGetProperty<int>(NamedProperties.Attachment.Size, out size) ? size.Value : 0;
            _ContentDisposition = Properties.TryGetProperty<string>(NamedProperties.Attachment.ContentDisposition, out stringProperty) ? stringProperty.Value : "";
            _MimeType = Properties.TryGetProperty<string>(NamedProperties.Attachment.MimeType, out stringProperty) ? stringProperty.Value : "";
        }

        /// <summary>
        /// Returns an <see cref="InfiniTec.Threading.AsyncOperation"/> which can be used to download the attachment.
        /// </summary>
        /// <param name="outputStream">The target stream for the content of the item.</param>
        /// <returns>An <see cref="InfiniTec.Threading.AsyncOperation"/>.</returns>
        public AsyncOperation<RequestStatus> GetDownloadOperation(Stream outputStream)
        {
            return Methods.Get(Url, outputStream);
        }

        /// <summary>
        /// Downloads the content of the attachment into the specified output stream. The current thread is blocked
        /// until the operation completes.
        /// </summary>
        /// <param name="outputStream">A <see cref="Stream"/> used to store the contents of the attachment.</param>
        /// <exception cref="AsyncOperationFailedException">Thrown, when the operation fails. See <see cref="Exception.InnerException"/> for details.</exception>
        public void Download(Stream outputStream)
        {
            using (AsyncOperation<RequestStatus> operation = GetDownloadOperation(outputStream))
            {
                operation.Run();

                operation.WaitForCompletion();
            }
        }

        /// <summary>
        /// Asynchronously downloads the content of the attachment into the specified output stream.
        /// </summary>
        /// <param name="outputStream">A <see cref="Stream"/> used to store the contents of the attachment.</param>
        public void DownloadAsync(Stream outputStream)
        {
            AsyncOperation<RequestStatus> operation;

            AsyncOperationSupport.InitiatePendingAsyncOperation();

            operation = GetDownloadOperation(outputStream);

            operation.ProgressChanged += DownloadProgressChanged;
            operation.Completed += Download_Completed;
            operation.Error += Download_Error;

            operation.Run();
        }

        void Download_Error(object sender, ExceptionEventArgs e)
        {
            AsyncOperationSupport.UnregisterPendingAsyncOperation();

            OnDownloadCompleted(new AsyncCompletedEventArgs(e.Exception, false, null));
        }

        void Download_Completed(object sender, CompletedEventArgs e)
        {
            AsyncOperationSupport.UnregisterPendingAsyncOperation();

            OnDownloadCompleted(new AsyncCompletedEventArgs(null, e.Status == OperationStatus.Aborted, null));
        }

        /// <summary>
        /// Executes the <see cref="DownloadCompleted"/> event.
        /// </summary>
        /// <param name="e">The event parameters.</param>
        protected virtual void OnDownloadCompleted(AsyncCompletedEventArgs e)
        {
            if (DownloadCompleted != null) DownloadCompleted(this, e);
        }
    }
}
